#!/usr/bin/env python

import sys
import time
import marshal
import types
import signal
import hmac 
import hashlib
import secret
try:
    from Crypto.Util import number
except ImportError:
    pass


TIMEOUT = 940
NCHALLENGES = 3
NSIZE = 1280


#!OBF_start
def create_key():
    if False: #used by obfuscation
        x = 7/0
        x = 7/0

    Nsize = NSIZE
    pqsize = Nsize/2
    N = 0 
    while(N.bit_length()!=Nsize):
        while True:
            p = number.getStrongPrime(pqsize)
            q = number.getStrongPrime(pqsize)
            if abs(p-q).bit_length() > (Nsize*0.496):
                break
        N = p*q
    phi = (p-1)*(q-1)

    limit1 = 0.261
    limit2 = 0.293
    while True:
        d = number.getRandomRange(pow(2,int(Nsize*limit1)),pow(2,int(Nsize*limit1)+1))
        while d.bit_length()<Nsize*limit2:
            ppp = 0
            while not number.isPrime(ppp):
                ppp = number.getRandomRange(pow(2,45),pow(2,45)+pow(2,12))
            d *= ppp
        if number.GCD(d, phi)!=1:
            continue
        e = number.inverse(d, phi)
        if number.GCD(e, phi)!=1:
            continue
        break

    zzz = 3

    return (N, e)
#!OBF_end



def send_challenges():

    #!OBF_call create_key
    
    ck = create_key

    challenges = []
    for _ in xrange(NCHALLENGES):
        n, e = ck()
        v = number.getRandomInteger(NSIZE-1)
        challenges.append((n, e, v))

    ctime = int(time.time())

    for c in challenges:
        n, e, v = c
        print n
        print e
        print v
    print ctime
    sys.stdout.flush()

    tohmac = repr((challenges, ctime))
    hm = hmac.new(secret.hmackey, msg=tohmac, digestmod=hashlib.sha256)
    print hm.hexdigest()
    sys.stdout.flush()


def test_challenges():
    sol = []
    for _ in xrange(NCHALLENGES):
        sol.append(int(sys.stdin.readline().strip()))

    challenges = []
    for _ in xrange(NCHALLENGES):
        n = int(sys.stdin.readline().strip())
        e = int(sys.stdin.readline().strip())
        v = int(sys.stdin.readline().strip())
        challenges.append((n, e, v))
    ctime = int(sys.stdin.readline().strip())
    givenhmac = sys.stdin.readline().strip().decode('hex')

    tohmac = repr((challenges, ctime))
    hm = hmac.new(secret.hmackey, msg=tohmac, digestmod=hashlib.sha256)
    if(not hmac.compare_digest(hm.digest(), givenhmac)):
        print "invalid hmac!"
        sys.stdout.flush()
        sys.exit(99)
    if(int(time.time()) - ctime > TIMEOUT):
        print "Too Slow!"
        sys.stdout.flush()
        sys.exit(12)

    print "Good..."
    sys.stdout.flush()

    for i in xrange(NCHALLENGES):
        n, e, v = challenges[i]
        s = sol[i]
        if pow(s, e, n) != v:
            print "Fail!"
            sys.stdout.flush()
            sys.exit(11)
            break
    else:
        print "Succcess!"
        with open("flag", "rb") as fp:
            flag = fp.read().strip()
            print flag
        sys.stdout.flush()



def main():
    print "Welcome to ASRybaB"
    sys.stdout.flush()
	
    sys.stdin.readline()

    print "1) get challenges"
    print "2) solve challenges"
    sys.stdout.flush()


    option = int(sys.stdin.readline().strip())
    if option == 1:
        send_challenges()
        sys.exit(0)
    elif option == 2:
        test_challenges()
        sys.exit(0)
    sys.exit(21)



if __name__ == "__main__":
    main()

